###############################################################
###   Tcl Variables
###############################################################
#set tclParams [list <param1> <value> <param2> <value> ... <paramN> <value>]
set tclParams [list place.closeImportedSites  1 \
                    hd.StrictContainRouting   1 \
              ]

#Define location for "Tcl" directory. Defaults to "./Tcl"
set tclHome "../Tcl"
if {[file exists $tclHome]} {
   set tclDir $tclHome
} elseif {[file exists "./Tcl"]} {
   set tclDir  "./Tcl"
} else {
   error "ERROR: No valid location found for required Tcl scripts. Set \$tclDir in design.tcl to a valid location."
}

###############################################################
### Define Part, Package, Speedgrade 
###############################################################
set device       "xc7z020"
set package      "clg484"
set speed        "-1"
set part         $device$package$speed

###############################################################
###  Setup Variables
###############################################################
####flow control
set run.topSynth   1
set run.oocSynth   1
set run.tdImpl     1
set run.oocImpl    0

####Report and DCP controls - values: 0-required min; 1-few extra; 2-all
set verbose      1
set dcpLevel     1

####Output Directories
set synthDir  "./Synth/zynq/OOC"
set implDir   "./Implement/zynq/OOC"
set dcpDir    "./Checkpoint/zynq/OOC"

####Input Directories
set srcDir     "./Sources"
set rtlDir     "$srcDir/hdl"
set prjDir     "$srcDir/prj"
set xdcDir     "$srcDir/xdc"

####Source required Tcl Procs
source $tclDir/design_utils.tcl
source $tclDir/synth_utils.tcl
source $tclDir/impl_utils.tcl
source $tclDir/hd_floorplan_utils.tcl

###############################################################
### Top Definition
###############################################################
set top "AEAD_Wrapper"
add_module $top
set_attribute module $top    top_level     1
set_attribute module $top    prj           $prjDir/$top.prj
set_attribute module $top    synth         ${run.topSynth}

####################################################################
### OOC Module Definition and OOC Implementation for each instance
####################################################################
set module1 "AEAD"
add_module $module1
set_attribute module $module1 prj          $prjDir/$module1.prj
set_attribute module $module1 synth        ${run.oocSynth}
#set_attribute module $module1 synth_options "-max_bram 0"

set instance "u_aead"
add_ooc_implementation $instance
set_attribute ooc $instance   module       $module1
set_attribute ooc $instance   inst         $instance
set_attribute ooc $instance   hierInst     $instance
set_attribute ooc $instance   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance   impl         ${run.oocImpl}
set_attribute ooc $instance   post_phys       		 0
set_attribute ooc $instance   preservation routing

####################################################################
### Create TopDown implementation run 
####################################################################
set module1File "$synthDir/$module1/${module1}_synth.dcp"
add_implementation TopDown
set_attribute impl TopDown      top          $top
set_attribute impl TopDown      implXDC      [list $xdcDir/${top}_flpn.xdc] 
set_attribute impl TopDown      td.impl      1
set_attribute impl TopDown      cores        [list $module1File                          \
                                                   [get_attribute module $top cores]     \
                                                   [get_attribute module $module1 cores] \
                                             ] 
set_attribute impl TopDown      impl         ${run.tdImpl}
set_attribute impl TopDown      route        0

# Build the designs
source $tclDir/run.tcl

exit