###############################################################
###   Tcl Variables
###############################################################
#set tclParams [list <param1> <value> <param2> <value> ... <paramN> <value>]
set tclParams [list place.closeImportedSites  1 \
                    hd.StrictContainRouting   1 \
              ]

#Define location for "Tcl" directory. Defaults to "./Tcl"
set tclHome "../Tcl"
if {[file exists $tclHome]} {
   set tclDir $tclHome
} elseif {[file exists "./Tcl"]} {
   set tclDir  "./Tcl"
} else {
   error "ERROR: No valid location found for required Tcl scripts. Set \$tclDir in design.tcl to a valid location."
}

###############################################################
### Define Part, Package, Speedgrade 
###############################################################
set device       "xc7vx485"
set package      "tffg1157"
set speed        "-2"
set part         $device$package$speed

###############################################################
###  Setup Variables
###############################################################
####flow control
set run.topSynth   1
set run.tdImpl     0

####Report and DCP controls - values: 0-required min; 1-few extra; 2-all
set verbose      1
set dcpLevel     1

####Output Directories
set synthDir  "./v7/TopDown/Synth"
set implDir   "./v7/TopDown/Implement"
set dcpDir    "./v7/TopDown/Checkpoint"

####Input Directories
set srcDir     "./Sources"
set rtlDir     "$srcDir/hdl"
set prjDir     "$srcDir/prj"
set xdcDir     "$srcDir/xdc"

####Source required Tcl Procs
source $tclDir/design_utils.tcl
source $tclDir/synth_utils.tcl
source $tclDir/impl_utils.tcl
source $tclDir/hd_floorplan_utils.tcl

###############################################################
### Top Definition
###############################################################
set top "CipherCore_Wrapper"
add_module $top
set_attribute module $top    top_level     1
set_attribute module $top    prj           $prjDir/$top.prj
set_attribute module $top    synth         ${run.topSynth}

####################################################################
### Create TopDown implementation run 
####################################################################
add_implementation TopDown
set_attribute impl TopDown      top          $top
set_attribute impl TopDown      implXDC      [list $xdcDir/${top}_flpn.xdc] 
set_attribute impl TopDown      td.impl      0
set_attribute impl TopDown      impl         ${run.tdImpl}
set_attribute impl TopDown      route        0

# Build the designs
source $tclDir/run.tcl

exit