###############################################################
###   Tcl Variables
###############################################################
#set tclParams [list <param1> <value> <param2> <value> ... <paramN> <value>]
set tclParams [list place.closeImportedSites  1 \
                    hd.StrictContainRouting   1 \
              ]

#Define location for "Tcl" directory. Defaults to "./Tcl"
set tclHome "../Tcl"
if {[file exists $tclHome]} {
   set tclDir $tclHome
} elseif {[file exists "./Tcl"]} {
   set tclDir  "./Tcl"
} else {
   error "ERROR: No valid location found for required Tcl scripts. Set \$tclDir in design.tcl to a valid location."
}

###############################################################
### Define Part, Package, Speedgrade 
###############################################################
set device       "xc7vx485"
set package      "tffg1157"
set speed        "-2"
set part         $device$package$speed

###############################################################
###  Setup Variables
###############################################################
####flow control
set run.topSynth   0
set run.oocSynth   0
set run.tdImpl     0
set run.oocImpl    1

####Report and DCP controls - values: 0-required min; 1-few extra; 2-all
set verbose      1
set dcpLevel     1

####Output Directories
set synthDir  "./Synth/v7/OOC"
set implDir   "./Implement/v7/OOC"
set dcpDir    "./Checkpoint/v7/OOC"

####Input Directories
set srcDir     "./Sources"
set rtlDir     "$srcDir/hdl"
set prjDir     "$srcDir/prj"
set xdcDir     "$srcDir/xdc"

####Source required Tcl Procs
source $tclDir/design_utils.tcl
source $tclDir/synth_utils.tcl
source $tclDir/impl_utils.tcl
source $tclDir/hd_floorplan_utils.tcl

###############################################################
### Top Definition
###############################################################
set top "AEAD_Wrapper"
add_module $top
set_attribute module $top    top_level     1
set_attribute module $top    prj           $prjDir/$top.prj
set_attribute module $top    synth         ${run.topSynth}

####################################################################
### OOC Module Definition and OOC Implementation for each instance
####################################################################
set module1 "AEAD"
add_module $module1
set_attribute module $module1 prj          $prjDir/$module1.prj
set_attribute module $module1 synth        ${run.oocSynth}
set_attribute module $module1 synth_options "-max_bram 0"

set instance "u_aead"
set instance1 "u_aead1"
add_ooc_implementation $instance1
set_attribute ooc $instance1   module       $module1
set_attribute ooc $instance1   inst         $instance1
set_attribute ooc $instance1   hierInst     $instance1
set_attribute ooc $instance1   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance1   impl         ${run.oocImpl}
set_attribute ooc $instance1   post_phys       		 0
set_attribute ooc $instance1   preservation routing
set_attribute ooc $instance1    phys         0

set instance2 "u_aead2"
add_ooc_implementation $instance2
set_attribute ooc $instance2   module       $module1
set_attribute ooc $instance2   inst         $instance2
set_attribute ooc $instance2   hierInst     $instance2
set_attribute ooc $instance2   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance2   impl         ${run.oocImpl}
set_attribute ooc $instance2   post_phys       		 0
set_attribute ooc $instance2   preservation routing
set_attribute ooc $instance2   opt_directive         Explore
set_attribute ooc $instance2   place_directive       Explore
set_attribute ooc $instance2   phys_directive        Explore
set_attribute ooc $instance2   route_directive       Explore

set instance3 "u_aead3"
add_ooc_implementation $instance3
set_attribute ooc $instance3   module       $module1
set_attribute ooc $instance3   inst         $instance3
set_attribute ooc $instance3   hierInst     $instance3
set_attribute ooc $instance3   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance3   impl         ${run.oocImpl}
set_attribute ooc $instance3   post_phys       		 1
set_attribute ooc $instance3   opt_directive         Explore
set_attribute ooc $instance3   place_directive       Explore
set_attribute ooc $instance3   phys_directive        Explore
set_attribute ooc $instance3   route_directive       Explore

set instance4 "u_aead4"
add_ooc_implementation $instance4
set_attribute ooc $instance4   module       $module1
set_attribute ooc $instance4   inst         $instance4
set_attribute ooc $instance4   hierInst     $instance4
set_attribute ooc $instance4   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance4   impl         ${run.oocImpl}
set_attribute ooc $instance4   post_phys       		 0
set_attribute ooc $instance4   preservation routing
set_attribute ooc $instance4   place_directive       WLDrivenBlockPlacement
set_attribute ooc $instance4   phys_directive        AggressiveExplore
set_attribute ooc $instance4   route_directive       MoreGlobalIterations

set instance5 "u_aead5"
add_ooc_implementation $instance5
set_attribute ooc $instance5   module       $module1
set_attribute ooc $instance5   inst         $instance5
set_attribute ooc $instance5   hierInst     $instance5
set_attribute ooc $instance5   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance5   impl         ${run.oocImpl}
set_attribute ooc $instance5   post_phys       		 0
set_attribute ooc $instance5   preservation routing
set_attribute ooc $instance5   place_directive       ExtraPostPlacementOpt
set_attribute ooc $instance5   route_directive       NoTimingRelaxation

set instance6 "u_aead6"
add_ooc_implementation $instance6
set_attribute ooc $instance6   module       $module1
set_attribute ooc $instance6   inst         $instance6
set_attribute ooc $instance6   hierInst     $instance6
set_attribute ooc $instance6   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance6   impl         ${run.oocImpl}
set_attribute ooc $instance6   post_phys       		 0
set_attribute ooc $instance6   preservation routing
set_attribute ooc $instance6   place_directive       WLDrivenBlockPlacement
set_attribute ooc $instance6   phys_directive        AlternateReplication
set_attribute ooc $instance6   route_directive       HigherDelayCost

set instance7 "u_aead7"
add_ooc_implementation $instance7
set_attribute ooc $instance7   module       $module1
set_attribute ooc $instance7   inst         $instance7
set_attribute ooc $instance7   hierInst     $instance7
set_attribute ooc $instance7   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance7   impl         ${run.oocImpl}
set_attribute ooc $instance7   post_phys       		 0
set_attribute ooc $instance7   preservation routing
set_attribute ooc $instance7   place_directive       LateBlockPlacement
set_attribute ooc $instance7   phys_directive        AggressiveExplore
set_attribute ooc $instance7   route_directive       HigherDelayCost

set instance8 "u_aead8"
add_ooc_implementation $instance8
set_attribute ooc $instance8   module       $module1
set_attribute ooc $instance8   inst         $instance8
set_attribute ooc $instance8   hierInst     $instance8
set_attribute ooc $instance8   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance8   impl         ${run.oocImpl}
set_attribute ooc $instance8   post_phys       		 0
set_attribute ooc $instance8   preservation routing
set_attribute ooc $instance8   place_directive       ExtraNetDelay_high
set_attribute ooc $instance8   route_directive       AdvancedSkewModeling

set instance9 "u_aead9"
add_ooc_implementation $instance9
set_attribute ooc $instance9   module       $module1

set_attribute ooc $instance9   inst         $instance9
set_attribute ooc $instance9   hierInst     $instance9
set_attribute ooc $instance9   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance9   impl         ${run.oocImpl}
set_attribute ooc $instance9   post_phys       		 0
set_attribute ooc $instance9   preservation routing
set_attribute ooc $instance9   place_directive       ExtraNetDelay_medium
set_attribute ooc $instance9   route_directive       AdvancedSkewModeling

set instance10 "u_aead10"
add_ooc_implementation $instance10
set_attribute ooc $instance10   module       $module1
set_attribute ooc $instance10   inst         $instance10
set_attribute ooc $instance10   hierInst     $instance10
set_attribute ooc $instance10   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance10   impl         ${run.oocImpl}
set_attribute ooc $instance10   post_phys       		 0
set_attribute ooc $instance10   preservation routing
set_attribute ooc $instance10   place_directive       ExtraNetDelay_low
set_attribute ooc $instance10   route_directive       AdvancedSkewModeling

set instance11 "u_aead11"
add_ooc_implementation $instance11
set_attribute ooc $instance11   module       $module1
set_attribute ooc $instance11   inst         $instance11
set_attribute ooc $instance11   hierInst     $instance11
set_attribute ooc $instance11   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance11   impl         ${run.oocImpl}
set_attribute ooc $instance11   post_phys       		 0
set_attribute ooc $instance11   preservation routing
set_attribute ooc $instance11   place_directive       SSI_ExtraTimingOpt
set_attribute ooc $instance11   route_directive       NoTimingRelaxation

set instance12 "u_aead12"
add_ooc_implementation $instance12
set_attribute ooc $instance12   module       $module1
set_attribute ooc $instance12   inst         $instance12
set_attribute ooc $instance12   hierInst     $instance12
set_attribute ooc $instance12   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance12   impl         ${run.oocImpl}
set_attribute ooc $instance12   post_phys       		 0
set_attribute ooc $instance12   preservation routing
set_attribute ooc $instance12   opt_directive         ExploreArea
set_attribute ooc $instance12   phys        		  0

set instance13 "u_aead13"
add_ooc_implementation $instance13
set_attribute ooc $instance13   module       $module1
set_attribute ooc $instance13   inst         $instance13
set_attribute ooc $instance13   hierInst     $instance13
set_attribute ooc $instance13   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance13   impl         ${run.oocImpl}
set_attribute ooc $instance13   post_phys       		 0
set_attribute ooc $instance13   preservation routing
set_attribute ooc $instance13   place_directive       ExtraPostPlacementOpt
set_attribute ooc $instance13   phys_directive        AddRetime
set_attribute ooc $instance13   route_directive       HigherDelayCost

#set instance14 "u_aead14"
#add_ooc_implementation $instance14
#set_attribute ooc $instance14   module       $module1
#set_attribute ooc $instance14   inst         $instance14
#set_attribute ooc $instance14   hierInst     $instance14
#set_attribute ooc $instance14   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
#set_attribute ooc $instance14   impl         ${run.oocImpl}
#set_attribute ooc $instance14   preservation routing
#set_attribute ooc $instance14   place_directive       SpreadLogic_high
#set_attribute ooc $instance14   phys_directive        AlternateReplication
#set_attribute ooc $instance14   route_directive       MoreGlobalIterations

set instance15 "u_aead15"
add_ooc_implementation $instance15
set_attribute ooc $instance15   module       $module1
set_attribute ooc $instance15   inst         $instance15
set_attribute ooc $instance15   hierInst     $instance15
set_attribute ooc $instance15   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance15   impl         ${run.oocImpl}
set_attribute ooc $instance15   post_phys       		 0
set_attribute ooc $instance15   preservation routing
set_attribute ooc $instance15   phys_directive        Explore

set instance16 "u_aead16"
add_ooc_implementation $instance16
set_attribute ooc $instance16   module       $module1
set_attribute ooc $instance16   inst         $instance16
set_attribute ooc $instance16   hierInst     $instance16
set_attribute ooc $instance16   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance16   impl         ${run.oocImpl}
set_attribute ooc $instance16   post_phys       		 1
set_attribute ooc $instance16   preservation routing
set_attribute ooc $instance16   phys_directive        Explore
set_attribute ooc $instance16   route_options         -tns_cleanup

set instance17 "u_aead17"
add_ooc_implementation $instance17
set_attribute ooc $instance17   module       $module1
set_attribute ooc $instance17   inst         $instance17
set_attribute ooc $instance17   hierInst     $instance17
set_attribute ooc $instance17   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance17   impl         ${run.oocImpl}
set_attribute ooc $instance17   post_phys       		 0
set_attribute ooc $instance17   preservation routing
set_attribute ooc $instance17   opt_directive         RuntimeOptimized
set_attribute ooc $instance17   place_directive       RuntimeOptimized
set_attribute ooc $instance17   phys                  0
set_attribute ooc $instance17   route_directive       RuntimeOptimized

set instance18 "u_aead18"
add_ooc_implementation $instance18
set_attribute ooc $instance18   module       $module1
set_attribute ooc $instance18   inst         $instance18
set_attribute ooc $instance18   hierInst     $instance18
set_attribute ooc $instance18   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance18   impl         ${run.oocImpl}
set_attribute ooc $instance18   post_phys       		 0
set_attribute ooc $instance18   preservation routing
set_attribute ooc $instance18   opt_directive         RuntimeOptimized
set_attribute ooc $instance18   place_directive       Quick
set_attribute ooc $instance18   phys                  0
set_attribute ooc $instance18   route_directive       Quick

set instance19 "u_aead19"
add_ooc_implementation $instance19
set_attribute ooc $instance19   module       $module1
set_attribute ooc $instance19   inst         $instance19
set_attribute ooc $instance19   hierInst     $instance19
set_attribute ooc $instance19   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance19   impl         ${run.oocImpl}
set_attribute ooc $instance19   post_phys       		 0
set_attribute ooc $instance19   preservation routing
set_attribute ooc $instance19   place_directive       SpreadLogic_high
set_attribute ooc $instance19   phys_directive        AlternateReplication
set_attribute ooc $instance19   route_directive       MoreGlobalIterations

set instance20 "u_aead20"
add_ooc_implementation $instance20
set_attribute ooc $instance20   module       $module1
set_attribute ooc $instance20   inst         $instance20
set_attribute ooc $instance20   hierInst     $instance20
set_attribute ooc $instance20   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance20   impl         ${run.oocImpl}
set_attribute ooc $instance20   post_phys       		 0
set_attribute ooc $instance20   preservation routing
set_attribute ooc $instance20   place_directive       SpreadLogic_medium
set_attribute ooc $instance20   phys_directive        AlternateReplication
set_attribute ooc $instance20   route_directive       NoTimingRelaxation

set instance21 "u_aead21"
add_ooc_implementation $instance21
set_attribute ooc $instance21   module       $module1
set_attribute ooc $instance21   inst         $instance21
set_attribute ooc $instance21   hierInst     $instance21
set_attribute ooc $instance21   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance21   impl         ${run.oocImpl}
set_attribute ooc $instance21   post_phys       		 0
set_attribute ooc $instance21   preservation routing
set_attribute ooc $instance21   place_directive       SpreadLogic_low
set_attribute ooc $instance21   route_directive       HigherDelayCost

set instance22 "u_aead22"
add_ooc_implementation $instance22
set_attribute ooc $instance22   module       $module1
set_attribute ooc $instance22   inst         $instance22
set_attribute ooc $instance22   hierInst     $instance22
set_attribute ooc $instance22   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance22   impl         ${run.oocImpl}
set_attribute ooc $instance22   post_phys       		 0
set_attribute ooc $instance22   preservation routing
set_attribute ooc $instance22   place_directive       SSI_SpreadSLLs
set_attribute ooc $instance22   route_directive       NoTimingRelaxation

set instance23 "u_aead23"
add_ooc_implementation $instance23
set_attribute ooc $instance23   module       $module1
set_attribute ooc $instance23   inst         $instance23
set_attribute ooc $instance23   hierInst     $instance23
set_attribute ooc $instance23   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance23   impl         ${run.oocImpl}
set_attribute ooc $instance23   post_phys       		 0
set_attribute ooc $instance23   preservation routing
set_attribute ooc $instance23   place_directive       SSI_BalanceSLLs
set_attribute ooc $instance23   route_directive       NoTimingRelaxation

set instance24 "u_aead24"
add_ooc_implementation $instance24
set_attribute ooc $instance24   module       $module1
set_attribute ooc $instance24   inst         $instance24
set_attribute ooc $instance24   hierInst     $instance24
set_attribute ooc $instance24   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance24   impl         ${run.oocImpl}
set_attribute ooc $instance24   post_phys       		 0
set_attribute ooc $instance24   preservation routing
set_attribute ooc $instance24   place_directive       SSI_BalanceSLRs

set instance25 "u_aead25"
add_ooc_implementation $instance25
set_attribute ooc $instance25   module       $module1
set_attribute ooc $instance25   inst         $instance25
set_attribute ooc $instance25   hierInst     $instance25
set_attribute ooc $instance25   implXDC      [list $xdcDir/${instance}_phys.xdc         \
                                                 $xdcDir/${instance}_ooc_timing.xdc   \
                                                 $xdcDir/${instance}_ooc_budget.xdc   \
                                                 $xdcDir/${instance}_ooc_optimize.xdc \
                                           ] 
set_attribute ooc $instance25   impl         ${run.oocImpl}
set_attribute ooc $instance25   post_phys       		 0
set_attribute ooc $instance25   preservation routing
set_attribute ooc $instance25   place_directive       SSI_HighUtilSLRs


####################################################################
### Create TopDown implementation run 
####################################################################
set module1File "$synthDir/$module1/${module1}_synth.dcp"
add_implementation TopDown
set_attribute impl TopDown      top          $top
set_attribute impl TopDown      implXDC      [list $xdcDir/${top}_flpn.xdc] 
set_attribute impl TopDown      td.impl      1
set_attribute impl TopDown      cores        [list $module1File                          \
                                                   [get_attribute module $top cores]     \
                                                   [get_attribute module $module1 cores] \
                                             ] 
set_attribute impl TopDown      impl         ${run.tdImpl}
set_attribute impl TopDown      route        0

# Build the designs
source $tclDir/run.tcl

exit